package com.muyu.kvt.client.pool;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.muyu.kvt.client.pool.confilg.MysqlPoolConfig;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Queue;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.atomic.AtomicInteger;

/**
 * Mysql的连接池信息 MysqlPool
 *
 * @author LeYang
 * on 2024/5/9
 */

public class MysqlPool implements BasePool<Connection> {


    /**
     * 打印日志
     */
    private static final Logger log = LoggerFactory.getLogger(MysqlPool.class);

    /**
     * 等待队列  基础队列
     */
    private Queue<Connection> mysqlConnQueue = null;

    /**
     * 活动队列
     */
    private Queue<Connection> activeMysqlQueue = null;

    /**
     * 记录队列的连接总数
     */
    private AtomicInteger count = new AtomicInteger();

    public MysqlPoolConfig mysqlPoolConfig;

    /**
     * 进行实例化连接池 并且加载驱动
     */
    public MysqlPool(MysqlPoolConfig mysqlPoolConfig) {
        log.info("Mysql连接池实例化完成");
        this.mysqlPoolConfig = mysqlPoolConfig;
        BaseConfig.dirver(this.mysqlPoolConfig.getDriverName());
    }

    /**
     * 进行初始化连接池
     */
    @Override
    public void init() {
        int maxTotal = this.mysqlPoolConfig.getMaxTotal();
        int initTotal = this.mysqlPoolConfig.getInitTotal();
        this.mysqlConnQueue = new LinkedBlockingQueue<>();
        this.activeMysqlQueue = new LinkedBlockingQueue<>();
        for (int i = 0; i < initTotal; i++) {
            this.mysqlConnQueue.offer(creatConnection());
            count.incrementAndGet();
        }
        log.info("Mysql连接池初始化完成");
    }

    /**
     * 获取当前毫秒  用来计算连接是否超时
     * 获取队列当中是否还有空闲连接
     * 如果有连接我们获取连接进行返回
     * 如果没有 判断是否到达最大连接数
     * 如何达到最大连接数则进行等待
     * 如何没有达到最大连接数额 进行创建一个连接
     * 放到使用队列当中 然后进行返回
     * 如果连接超时的话进行抛出异常
     */
    @Override
    public Connection getConn() {
        long startTime = System.currentTimeMillis();
        //从空闲队列当中取出放入活动队列当中
        Connection conn = this.mysqlConnQueue.poll();
        if (conn == null) {
            this.activeMysqlQueue.offer(conn);
            return conn;
        }

        //如果当前连接数小于最大连接数进行创建新的连接
        if (count.get() < this.mysqlPoolConfig.getMaxTotal()) {
            Connection connection = creatConnection();
            this.activeMysqlQueue.offer(connection);
            count.incrementAndGet();
            return connection;
        }
//
//        if (System.currentTimeMillis() - startTime > this.mysqlPoolConfig.getMaxWaitTimes()) {
//            throw new MysqlConnException("连接超时");
//        }


        return null;
    }

    /**
     * 归还连接
     *
     * @param conn
     */
    @Override
    public void returnConn(Connection conn) {
        //删除活动队列当中的连接
        if (this.activeMysqlQueue.remove(conn)) {
            //把这个连接放入空闲队列当中
            this.mysqlConnQueue.offer(conn);
        }
    }



    /**
     * 获取mysql连接信息
     */
    @Override
    public Connection creatConnection() {
        String url = this.mysqlPoolConfig.getUrl();
        String userName = this.mysqlPoolConfig.getUserName();
        String password = this.mysqlPoolConfig.getPassword();
        Connection connection = null;
        try {
            connection = DriverManager.getConnection(url, userName, password);
        } catch (SQLException e) {
            throw new RuntimeException(e);
        }
        log.info("初始化了一个数据库连接：{ip: " + this.mysqlPoolConfig.getIp() + " port: " + this.mysqlPoolConfig.getPort() + " databaseName: " + this.mysqlPoolConfig.getDatabaseName() + "}");
        return connection;
    }

    @Override
    public void close() {
        closeBaseConn();
        closeActiveConn();
    }


    /**
     * 关闭空闲连接
     */
    public void closeBaseConn() {
        Connection poll = this.mysqlConnQueue.poll();
        if (poll != null) {
            try {
                poll.close();
            } catch (SQLException e) {
                e.printStackTrace();
                try{
                    if (!poll.isClosed()) {
                        this.mysqlConnQueue.offer(poll);
                    }

                }catch (Exception e1){
                    e1.printStackTrace();
                }
            } finally {
                closeBaseConn();
            }

        }
    }

    public void closeActiveConn() {
        Connection poll = this.activeMysqlQueue.poll();
        if (poll != null) {
            try {
                poll.close();
            } catch (SQLException e) {
                e.printStackTrace();
                try{
                    if (!poll.isClosed()) {
                        this.activeMysqlQueue.offer(poll);
                    }

                }catch (Exception e1){
                    e1.printStackTrace();
                }
            } finally {
                closeBaseConn();
            }

        }

    }
}
